using Jose;

namespace FitConnect.Encryption;

public class JoseEncryptor : IEncryptor {
    private string Encrypt(Jwk key, string plain) {
        return JWE.Encrypt(plain,
            new JweRecipient[] { new JweRecipient(JweAlgorithm.RSA_OAEP, key) },
            JweEncryption.A256GCM, compression: JweCompression.DEF);
    }

    private (string cypher, byte[] tag) Decrypt(Jwk key, string payload) {
        var result = JWE.Decrypt(payload, key);

        return (result.Plaintext, result.AuthTag);
    }

    public string Encrypt(string key, string plain) {
        var jwk = Jwk.FromJson(key, new Jose.JsonMapper());
        return Encrypt(jwk, plain);
    }

    public string Decrypt(string key, string cipher) {
        var jwk = Jwk.FromJson(key, new Jose.JsonMapper());
        return Decrypt(jwk, cipher).cypher;
    }

    public string Encrypt(string key, byte[] plain) {
        var jwk = Jwk.FromJson(key, new Jose.JsonMapper());
        return JWE.EncryptBytes(plain,
            new JweRecipient[] { new JweRecipient(JweAlgorithm.RSA_OAEP, key) },
            JweEncryption.A256GCM, compression: JweCompression.DEF);
    }
}
