using System;
using System.IO;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using FitConnect;
using FitConnect.Encryption;
using FluentAssertions;
using Microsoft.Extensions.Logging;
using NUnit.Framework;

namespace SenderTest;

[Ignore("Encryption strategy does not match the one used on server")]
public class SenderEncryptionWithSelfSignedCertificateTest {
    private const string ToEncrypt = "This is a test message";
    private static byte[]? cypher;
    private X509Certificate2 _certificate = null!;
    private ILogger<SenderEncryptionWithSelfSignedCertificateTest> _logger = null!;
    private Sender _sender = null!;


    /*
     * Encryption test must be changed for production to only allow extern signed certificates
     * and forbid self-signed certificates.
     */

    [OneTimeSetUp]
    public void OneTimeSetup() {
    }

    [OneTimeTearDown]
    public void OneTimeTearDown() {
        _certificate.Dispose();
        // File.Copy("./certificate.pfx", "../../../Certificates/certificate.pfx", true);
        // File.Copy("./certificate.cer", "../../../Certificates/certificate.cer", true);
        File.Delete("./certificate.pfx");
        File.Delete("./certificate.cer");
    }

    [SetUp]
    public void Setup() {
        _logger = LoggerFactory.Create(cfg => cfg.AddConsole())
            .CreateLogger<SenderEncryptionWithSelfSignedCertificateTest>();
        _sender = Sender.Create(
            FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development),
            _logger);
    }


    [Test]
    [Order(10)]
    public void CryptWithOutPublicKeyImport() {
        cypher = _sender.Encryption.Encrypt(Encoding.UTF8.GetBytes(ToEncrypt));

        _logger.LogInformation("Cypher: {}", Convert.ToBase64String(cypher));
    }

    [Test]
    [Order(20)]
    public void Decrypt_ResultShouldMatchToEncrypt() {
        var result = _sender.Encryption.Decrypt(cypher!);
        Encoding.UTF8.GetString(result).Should().Be(ToEncrypt);
    }


    
}
