using System.Net;
using System.Net.Http.Json;
using System.Text;

namespace FitConnect.RestService;

public interface IRestCallService {
    public IWebProxy? Proxy { get; set; }
}

public abstract class RestCallService {
    private readonly string _baseUrl;
    public IWebProxy? Proxy { get; set; }

    protected RestCallService(string baseUrl) {
        _baseUrl = baseUrl;
    }

    protected HttpClient CreateClient() {
        var clientHandler = new HttpClientHandler() {
            Proxy = Proxy
        };
        var client = new HttpClient(handler: clientHandler);
        client.DefaultRequestHeaders.Add("Accept", "application/json");
        return client;
    }

    internal async Task<T?> RestCall<T>(Uri uri, HttpMethod method, string body) {
        var client = CreateClient();

        var request = new HttpRequestMessage();
        request.Method = method;
        request.Content = new StringContent(body, Encoding.UTF8, "application/json");
        request.RequestUri = uri;
        var response = await client.SendAsync(request);

        if (response.IsSuccessStatusCode)
            return await response.Content.ReadFromJsonAsync<T>();

        throw new HttpRequestException("Error calling FitConnect API");
    }
}
